from logan.runner import run_app



CONFIG_TEMPLATE = """
import os.path

CONF_ROOT = os.path.dirname(__file__)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(CONF_ROOT, 'eukalypse_now.db'),
        'USER': 'postgres',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

# Mail server configuration

# For more information check Django's documentation:
#  https://docs.djangoproject.com/en/1.3/topics/email/?from=olddocs#e-mail-backends

EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'

EMAIL_HOST = 'localhost'
EMAIL_HOST_PASSWORD = ''
EMAIL_HOST_USER = ''
EMAIL_PORT = 25
EMAIL_USE_TLS = False


SITE_URL = 'http://localhost:8000'
MEDIA_URL = SITE_URL + '/media/'
MEDIA_ROOT=''

SENTRY_DSN = ''

if SENTRY_DSN:
    LOGGING['handlers']['sentry'] ={
            'level': 'INFO',
            'class': 'raven.handlers.logging.SentryHandler',
            'dsn': SENTRY_DSN,
        }
    LOGGING['loggers']['django.request']['handlers'].append('sentry')

"""


def generate_settings():
    """
    This command is run when ``default_path`` doesn't exist, or ``init`` is
    run and returns a string representing the default data to put into their
    settings file.
    """
    return CONFIG_TEMPLATE


def main():
    run_app(
        project='eukalypse_now',
        default_config_path='~/.eukalypse_now/eukalypse_now.conf.py',
        default_settings='eukalypse_now.settings.defaults',
        settings_initializer=generate_settings,
        settings_envvar='EUKALYPSE_NOW_CONF',
    )

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# 
from celery.task import  task, Task
from django.core.management import call_command
from models import Project
import  logging
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from eukalypse_now.models import Project
from eukalypse_now.models import Testresult
from eukalypse_now.models import Testrun
from eukalypse.eukalypse import Eukalypse
import os
from datetime import datetime
import smtplib
from email.mime.text import MIMEText


logger = logging.getLogger('eukalypse-now.task')

class PeriodicalCheck(Task):
    """ Task to automatically check all the active projects and run the testruns one after another."""
    def run(self, **kwargs):
        logger.debug("run PeriodicalCheck")
        for project in Project.objects.filter(active=True):
            logger.error(project)
            call_command("testrun", project.name)


class SetImageFromUrl(Task):

    def run(self, instance, **kwargs):
        logger.debug("run SetImageFromUrl")

        instance._set_image_from_url()


class Testrunner(Task):
    def _getMediaUrl(self, path):
        return path[len(settings.MEDIA_ROOT):]

    def run(self, arg_project = None, *args, **options):
        project =  Project.objects.get(name=arg_project)
        testrun = Testrun.objects.create(project=project)
        for test in project.tests.all():

            e = Eukalypse()
            e.browser = settings.EUKALYPSE_BROWSER
            e.host = settings.EUKALYPSE_HOST
            e.wait=test.wait
            e.output = os.path.join(settings.MEDIA_ROOT , 'images')
            eukalypse_result_object = e.compare(test.get_identifier(), test.image, test.url)
            e.disconnect()
            if eukalypse_result_object.clean:
                testresult = Testresult.objects.create(\
                    test=test, \
                    testrun = testrun, \
                    error = False, \
                    resultimage=self._getMediaUrl(eukalypse_result_object.target_img),\
                    referenceimage=test.image, \
                    )
            else:
                testresult = Testresult.objects.create(\
                    test=test, \
                    testrun = testrun, \
                    error = True, \
                    resultimage=self._getMediaUrl(eukalypse_result_object.target_img), \
                    referenceimage=test.image, \
                    errorimage=self._getMediaUrl(eukalypse_result_object.difference_img), \
                    errorimage_improved=self._getMediaUrl(eukalypse_result_object.difference_img_improved)\
                    )
                testrun.error = True

            testresult.save()

        testrun.save()

        if project.notify_mail:
            if (project.notify_only_error and testrun.error) or not project.notify_only_error:
                from django.template.loader import render_to_string
                render = render_to_string('eukalypse_now/mail.html', {'testrun': testrun, 'SITE_URL': settings.SITE_URL})
                msg = MIMEText(render, 'html')
                msg['Subject'] = 'Pixel Reporting'
                msg['From'] = settings.NOTIFY_MAIL_SENDER
                msg['To'] = project.notify_recipient
                s = smtplib.SMTP(settings.EMAIL_HOST)
                rcpts = [r.strip() for r in  project.notify_recipient.split(',') if r]
                s.sendmail(settings.NOTIFY_MAIL_SENDER, rcpts, msg.as_string())
                s.quit()






# -*- coding: utf-8 -*-
import os
from django.contrib import admin
from django.conf import settings
# Eukalypse Imports
from models import Test, Project, Testrun, Testresult
from eukalypse.eukalypse import Eukalypse


class TestInline(admin.StackedInline):
    model = Test
    extra = 0

class ProjectAdmin(admin.ModelAdmin):
    fieldsets = [
        (None,               {'fields': ['name', 'active']}),
        ('Notification Mail',               {'fields': ['notify_mail', 'notify_only_error', 'notify_recipient']}),
    ]
    inlines = [TestInline]
    list_display = ('name', 'active', )
    def save_formset(self, request, form, formset, change):
        """ Speichert Screenshots der tests als Referenz, wenn noch kein Bild Hinterlegt.
        Damit das auch beim anlegen neuer Tests und speichern bestehnder funktioniert
        wird jedesmal über das formset iteriert und in jedem Test der kein Bild hat 
        das bild neu erzeugt und gespeichert.
        """
        formset.save()
        for f in formset.forms:
            if not f.instance.image:
                #the default images has some advantages to a "None" Image. the most important one is: 
                #if you save twice and the first save is not finished creating the image, the second save will not trigger another image creation because an image - the default one- is already set. 
                f.instance._set_image_default()
                from eukalypse_now.tasks import SetImageFromUrl 
                SetImageFromUrl.delay(f.instance)


admin.site.register(Project, ProjectAdmin)


admin.site.register(Test)
admin.site.register(Testrun)
admin.site.register(Testresult)




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Project.active'
        db.add_column('eukalypse_now_project', 'active',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Project.active'
        db.delete_column('eukalypse_now_project', 'active')


    models = {
        'eukalypse_now.project': {
            'Meta': {'object_name': 'Project'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'notify_mail': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'notify_only_error': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'notify_recipient': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        'eukalypse_now.test': {
            'Meta': {'object_name': 'Test'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.SlugField', [], {'max_length': '200'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'tests'", 'to': "orm['eukalypse_now.Project']"}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'eukalypse_now.testresult': {
            'Meta': {'object_name': 'Testresult'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'error': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'errorimage': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'errorimage_improved': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'referenceimage': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'resultimage': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'test': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['eukalypse_now.Test']"}),
            'testrun': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testresult'", 'to': "orm['eukalypse_now.Testrun']"})
        },
        'eukalypse_now.testrun': {
            'Meta': {'object_name': 'Testrun'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'error': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testrun'", 'null': 'True', 'to': "orm['eukalypse_now.Project']"})
        }
    }

    complete_apps = ['eukalypse_now']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Project'
        db.create_table('eukalypse_now_project', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('eukalypse_now', ['Project'])

        # Adding model 'Test'
        db.create_table('eukalypse_now_test', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('project', self.gf('django.db.models.fields.related.ForeignKey')(related_name='tests', to=orm['eukalypse_now.Project'])),
            ('identifier', self.gf('django.db.models.fields.SlugField')(max_length=200)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('eukalypse_now', ['Test'])

        # Adding model 'Testresult'
        db.create_table('eukalypse_now_testresult', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('test', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['eukalypse_now.Test'])),
            ('testrun', self.gf('django.db.models.fields.related.ForeignKey')(related_name='testresult', to=orm['eukalypse_now.Testrun'])),
            ('resultimage', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('error', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('referenceimage', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('errorimage', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('errorimage_improved', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('eukalypse_now', ['Testresult'])

        # Adding model 'Testrun'
        db.create_table('eukalypse_now_testrun', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('project', self.gf('django.db.models.fields.related.ForeignKey')(related_name='testrun', null=True, to=orm['eukalypse_now.Project'])),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('eukalypse_now', ['Testrun'])


    def backwards(self, orm):
        # Deleting model 'Project'
        db.delete_table('eukalypse_now_project')

        # Deleting model 'Test'
        db.delete_table('eukalypse_now_test')

        # Deleting model 'Testresult'
        db.delete_table('eukalypse_now_testresult')

        # Deleting model 'Testrun'
        db.delete_table('eukalypse_now_testrun')


    models = {
        'eukalypse_now.project': {
            'Meta': {'object_name': 'Project'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'eukalypse_now.test': {
            'Meta': {'object_name': 'Test'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.SlugField', [], {'max_length': '200'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'tests'", 'to': "orm['eukalypse_now.Project']"}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'eukalypse_now.testresult': {
            'Meta': {'object_name': 'Testresult'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'error': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'errorimage': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'errorimage_improved': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'referenceimage': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'resultimage': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'test': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['eukalypse_now.Test']"}),
            'testrun': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testresult'", 'to': "orm['eukalypse_now.Testrun']"})
        },
        'eukalypse_now.testrun': {
            'Meta': {'object_name': 'Testrun'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testrun'", 'null': 'True', 'to': "orm['eukalypse_now.Project']"})
        }
    }

    complete_apps = ['eukalypse_now']


from settings.defaults import *


EUKALYPSE_BROWSER='chrome'
EUKALYPSE_HOST='http://192.188.23.18:4444'

EMAIL_HOST = 'mail.s-v.de'

#SENTRY_DSN = 'http://10eb8cf305f741e9acf227a82f12d21a:017e859ff4434ae496b001f5d1f638de@sentry.inhouse.s-v.de:9000/12'

if SENTRY_DSN:
    LOGGING['handlers']['sentry'] ={
            'level': 'INFO',
            'class': 'raven.handlers.logging.SentryHandler',
            'dsn': SENTRY_DSN,
        }
    LOGGING['loggers']['django.request']['handlers'].append('sentry')



from django.conf.urls import patterns, include, url
from django.conf import settings
from django.contrib.staticfiles.urls import staticfiles_urlpatterns


# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'eukalypse_now.views.home', name='home'),
    url(r'^testrun/list/', 'eukalypse_now.views.testrun_list'),
    url(r'^testrun/detail/(?P<testrun_id>\d+)/', 'eukalypse_now.views.testrun_detail'),
    url(r'^testresult/as_reference/(?P<testresult_id>\d+)/', 'eukalypse_now.views.testresult_as_reference'),
    url(r'^testresult/acknowledge_error/(?P<testresult_id>\d+)/', 'eukalypse_now.views.testresult_acknowledge_error'),
    url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT}),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^favicon\.ico$', 'django.views.generic.simple.redirect_to', {'url': '/static/favicon.ico'}),
    url(r'^$', 'eukalypse_now.views.testrun_list'),
)
urlpatterns += staticfiles_urlpatterns()



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings


class Command(BaseCommand):
    def handle(self, arg_project = None, *args, **options):
        from eukalypse_now.tasks import Testrunner 
        Testrunner.delay(arg_project)



#!/usr/bin/env python
"""
eukalypse_now

web server for eukalypse. 

:copyright: (c) 2012 Dennis Schwertel, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

from setuptools import setup, find_packages


tests_require = [
]

dependency_links = [
    'http://github.com/kinkerl/eukalypse/tarball/master#egg=eukalypse',
    'https://github.com/dcramer/logan/tarball/master#egg=logan'
]


install_requires = [
    'South==0.7.6',
    'Sphinx==1.1.3',
    'Pillow==1.7.7',
    'raven==2.0.7.1',
    'logan==0.5.1',
    'gunicorn==0.15.0',
    'eukalypse',
    'easy-thumbnails==1.1',
    'Django==1.4.1',
#    'kombu==2.4.8',
    'django-celery==3.0.11',
    'celery==3.0.12',

]

setup(
    name='eukalypse_now',
    version='0.1',
    author='Dennis Schwertel',
    author_email='s@digitalkultur.net',
    description='eukalypse web server',
    long_description=__doc__,
    package_dir={'': 'src'},
    packages=find_packages('src'),
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    dependency_links = dependency_links,
    license='BSD',
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'eukalypse_now = eukalypse_now.utils.runner:main',
        ],
    },
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)



from django.shortcuts import render
from models import Testrun, Testresult, Project
from django.shortcuts import get_object_or_404
from django.utils import simplejson
from django.http import HttpResponse

def testrun_list(request):
    projects = Project.objects.filter(active=True)
    return render(request, 'eukalypse_now/testrun/list.html', {"projects": projects})

def testrun_detail(request, testrun_id):
    testrun = get_object_or_404(Testrun, pk=testrun_id)
    return render(request, 'eukalypse_now/testrun/detail.html', {"testrun": testrun})

def testresult_as_reference(request, testresult_id):
    testresult = get_object_or_404(Testresult, pk=testresult_id)
    testresult.become_reference()
    to_json = {"return": "clear"}
    return HttpResponse(simplejson.dumps(to_json), mimetype="application/json")
    
    

def testresult_acknowledge_error(request, testresult_id):
    testresult = get_object_or_404(Testresult, pk=testresult_id)
    testresult.acknowledge_error()
    to_json = {"return": "clear"}
    return HttpResponse(simplejson.dumps(to_json), mimetype="application/json")
    
    
    



# -*- coding: utf-8 -*-
import os
from django.db import models
from django.conf import settings
from datetime import datetime
from eukalypse.eukalypse import Eukalypse


class Project(models.Model):
    name = models.CharField(max_length=200)
    active = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    notify_mail = models.BooleanField(default=False, help_text="send notification mail after a testrun")
    notify_only_error = models.BooleanField(default=False, help_text="only send the mail if an error occurs")
    notify_recipient =  models.TextField(blank=True, null=True, help_text="Multiple recipient are ', ' (comma space) separated: mail@domain.com, mail2@domain.com")
    
    def __unicode__(self):
        return self.name


    def list_testrun_for_overview(self):
        return self.testrun.order_by('-created')[:5]
    
    
    
class Test(models.Model):
    project = models.ForeignKey('Project', related_name='tests')
    identifier = models.SlugField(max_length=200)
    url = models.URLField()
    wait = models.IntegerField(default=0, help_text="Wait xx seconds after the website finished loading to create the screenshot.")
    image = models.ImageField(upload_to='images', null=True, blank=True)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return u"%s <%s>" % (self.identifier, self.image)

    def get_identifier(self):
        from django.template.defaultfilters import slugify
        """Liefert einen identifier String für dieses Screenshotbild."""
        return u"%s" % slugify(self.identifier + "-" + str(datetime.now()).replace(' ', '-'))

    def _set_image_default(self):
        self.image = "images/default.png"
        self.save()

    def _set_image_from_url(self):
        """Erzeugt Screenshot von url mit Eukalypse und speichert selbiges image (überschreibt evtl vorhandenes image)."""
        identifier = self.get_identifier() # Vorher speichern!
        e = Eukalypse()
        e.browser = settings.EUKALYPSE_BROWSER
        e.host = settings.EUKALYPSE_HOST
        e.wait = self.wait
        e.output = os.path.join(settings.MEDIA_ROOT , 'images')
        e.screenshot(identifier, self.url)
        self.image = "images/" + identifier + ".png"
        self.save()

class Testresult(models.Model):
    test = models.ForeignKey('Test')
    testrun = models.ForeignKey('Testrun',  related_name='testresult')
    resultimage = models.ImageField(upload_to='images', null=True, blank=True)
    error = models.BooleanField(default=False)
    error_acknowledged = models.BooleanField(default=False)
    error_reference_updated = models.BooleanField(default=False)
    referenceimage = models.ImageField(upload_to='images', null=True, blank=True)
    errorimage = models.ImageField(upload_to='images', null=True, blank=True)
    errorimage_improved = models.ImageField(upload_to='images', null=True, blank=True)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return "{0}-{1}".format(self.test.identifier, self.testrun.created)

    def become_reference(self):
        self.test.image = self.resultimage
        self.test.save()
        self.error_reference_updated=True
        self.save()
        
    def get_become_reference_url(self):
        return "/testresult/as_reference/{0}".format(self.id)
        
    def acknowledge_error(self):
        self.error_acknowledged=True
        self.save()
    
    def get_acknowledge_error_url(self):
        return "/testresult/acknowledge_error/{0}".format(self.id)

class Testrun(models.Model):
    project = models.ForeignKey('Project', null=True,  related_name='testrun')
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    error = models.BooleanField(default=True)
    def __unicode__(self):
        return "{0}-{1}".format(self.project.name, self.created)

    def get_absolute_url(self):
        return "/testrun/detail/%i/" % self.id
